 function[AA,BB,CC,DD]=white2(ao,c,sy,D,R)
% function[AA,BB,CC,DD]=white2(ao,c,sy,D,R)
%   Creates the state space system [AA,BB,CC,DD] that accepts
%   v(t+1) as in input and puts out the innovation u(t) to
%   y(t) as an output.  The program couples the systems
%
%        x(t+1) = ao * x(t) + c * w(t+1)
%          y(t) = sy * x(t)  + v(t)
%          v(t) = D * v(t-1) + eta(t)
%
%    where E eta(t) eta(t)' = R.
%
%        xh(t+1) = (ao-k1*GG) * xh(t) + k1 * y(t)
%          u(t)  = - GG * xh(t) + u(t)
%
%    where w(t+1) is the innovation to agents' info sets,
%    eta(t) is the innovation to measurement error, and where
%    u(t) is the fundamental (Wold) representation innovation.
%
%    The (minimum-realization) state space system [AA,BB,CC,DD]
%    for the coupled system is returned.
%    To compute the impulse response function, use
%    dimpulse.

   Q=c*c';
   G=sy;
   [m,n]=size(sy);
   Vm=eye(m);
   Vmm=zeros(m);
   GG=G*ao-D*G;
   Q1=Q;
   W1=Q*G';
   R1=R+G*Q*G';
   [k1,s1]=kfilter(ao,GG,Q1,R1,W1);
%   V1=R1+GG*s1*GG';

[m1,n1]=size(c);
[m2,n2]=size(sy);
c=zeros(m1,m2);
% d=G*c;
d=eye(m2);     %  see equation 7.23
aa=ao-k1*GG;
bb=k1;cc=-GG;
[m1,n1]=size(bb);
[m2,n2]=size(GG);
dd=eye(m2);

[Am,Bm,Cm,Dm]=series(ao,c,GG,d,aa,bb,cc,dd);
[AA,BB,CC,DD]=minreal(Am,Bm,Cm,Dm);
%y1=dimpulse(AA,BB,CC,DD,1,20)
%y2=dimpulse(AA,BB,CC,DD,2,20)

