										
function[F,P,PT]=doublex3(A,B,Q,R,c,sig)
%function[F,P,PT]=doublex3(A,B,Q,R,c,sig)
%  This program uses the "doubling algorithm" to solve the
%  Riccati matrix difference equations associated with the
%  undiscounted quadratic-Gauusian linear optimal control problems.
%  The control problem has the form
%
%    S(t) = min over u(t) of
%    x(t)' Q x(t) + u(t)' R u(t) + (2/sig)log E(t) exp (sig/2) S(x(t+1)),
%
%  subject to 
%
%     x(t+1) = A' x(t) + c' u(t) + c w(t+1),
% 
%  where w(t+1) is a Gaussian martingale difference sequence 
%  with unit covariance matrix. The program returns the steady
%  state  value function in S.  The optimal control law is
%  u(t) = -F * x(t).  The program also returns ST, which
%  is the quadratic form in E(t) exp S(x(t+1)).
%  The matrices P and PT should be positive semi-definite

po=eye(size(A)); %  sets the initial penalty matrix (must be pos def)

ss=max(size(A));
v=eye(ss);

J=B*(R\B')-sig*c*c';          %  This is the J of Whittle.

fact=inv(v+J*po);

a0=fact*A;
b0=fact*J;
g0=Q-po+A'*po*fact*A;

tol=1e-15;
dd=1;
ss=max(size(A));
v=eye(ss);
										
while dd>tol
										
a1=a0*((v+b0*g0)\a0);
b1=b0+a0*((v+b0*g0)\(b0*a0'));
g1=g0+a0'*g0*((v+b0*g0)\a0);

%g1t=(v-sig*g1*c*c')\g1;
%g0t=(v-sig*g0*c*c')\g0;
%f1=(B*g1t*B'+R)\(B'*g1t*A);
%f0=(B*g0t*B'+R)\(B'*g0t*A);
dd=max(max(abs(g1-g0)));
a0=a1;
b0=b1;
g0=g1;
end
P=g1+po;
PT=inv(inv(P)-sig*c*c');
F=(R+B'*PT*B)\B'*PT*A;

