function[K1,K2,S] =basar(A,D,C,E,H,sig)
%function[K1,K2,S] =basar(A,D,C,E,H,sig)
%
%
% Does robust filter for the Basar-Bernhard setup
%
%  x_{t+1} = A x_t + D w_t
%  y_t = C x_t + E w_t
%  z_t = H x_t 
%
%  set H'*H = Q, E*E' = N 
%  and iterate on
% 
%  S_{j+1} = A(S_j^{-1} + C'N^{-1}C + sig*Q)^{-1} *A' + D*D';
%
%  compute 
%  con=A(I + S(C'N^{-1}C + sig Q)^{-1} S
%  K1 = con*C'N^{-1}
%  K2 = con*sig*Q
%
%  \hat x_{t+1} = A hat x_t + K1 *(y - C \hat x_t) + K2 \hat x_t 
%
%  remember the sign switch in doublex4.m because the filtering
%  problem is a min problem.  So we have to do this here too.
%    remember that doublex.m solves a MIN problem, which requires
%    us to reverse the sign convention for sig
%  this is why we put + sig where Basar has - gamma^_{-2}
Q=H'*H;
N=E*E';


tol=.00001;
dd=10;

na=length(A);

S0=eye(na)*.001;

while dd>tol
   
   S1=A*(inv(inv(S0)+ (C'/N)*C + sig*Q))*A+ D*D';
   
   dd=norm(S1-S0)/norm(S0);
   
   S0=S1;
end
S=S1;
con=A*inv(eye(na)+S*((C/N)*C+sig*Q))*S;
K1=con*(C/N);
K2=-con*sig*Q;    %  again, -sig = gamma^{-2}
  