% casey1.m
%  modified version of solvdist.m, designed for checking purposes
%  a script file (not a function), finds a competitive equilibrium
%   for a representative agent economy with distortions.  Agents maximize:
%
%      -.5 SUM beta^t [(s(t)-b(t)).(s(t)-b(t)) + l(t)^2]
%  subject to    g(t).g(t) = l(t)^2
%                   z(t+1) = a22 * z(t) + c2 * w(t+1)
%                     b(t) = ub * z(t), d(t) = ud * z(t), E(t) = ue * z(t)
%  h(t) =  deltah * h(t-1) +  deltaH * H(t-1) + thetah * c(t) + thetaH * C(t)
%  s(t) = lambda * h(t-1) + lambda * H(t-1) +    pic * c(t) +piC * C(t)
%  k(t) =  deltak * k(t-1) +  thetak * i(t)
%  SUM beta^t [(I+tauc)*p(t).c(t) + (I+taui)*q(t).i(t) - (1-taul)*w(t).g(t)
%     - alpha(t).(d(t) + gammaK*K(t-1)) - (I-tauk)*r(t).k(t-1) - T(t)]  =  0
%
% Firms maximize profits:
% SUM beta^t [p(t).(c(t)+E(t))+q(t).i(t)-r(t).k(t-1)-alpha(t).d(t)-w(t)*g(t)]
% subject to              g(t).g(t)  =  l(t)^2
% phic*(c(t)+E(t)) +phig*g(t) +phii*i(t) = gammak*k(t-1) +gammaK*K(t-1) +d(t)
%
%   Where x(t) = [h(t-1)',k(t-1)',z(t)']', the solution of the problem is
%      x(t+1) = ao * x(t) + c * w(t+1)
%        j(t) = sj * x(t),  j = k, h, c, e, s, g, i, b, d, p, q, w, r, alpha
%   The program also computes the household's Lagrange multipliers
%        muj = mj * x(t)         for j = k, h, s, and z.   (mu0 set to 1)

%   Casey B. Mulligan  3/26/92


g=1+pic^2/phi1^2;
a3=(ub-pic*ud)/g;
a1=-lambda/g;
a2=-pic*gamma/g;
a4=-(pic/(phi1^2))*thetah/g;
a5=pic*thetak/((phi1^2)*g);

a6=-pic*a1/phi1^2;
a7=-pic*a2/phi1^2;
a8=-pic*a3/phi1^2;
a9=-pic*a4/phi1^2 - thetah/phi1^2;
a10=-pic*a5/phi1^2 + thetak/phi1^2;

a11=-a6;
a12=gamma-a7;
a13=-a8+ud;
a14=-a9;
a15=-a10;

a16=thetak*a6;
a17=thetak*a7 + deltak;
a18= thetak*a8;
a19= thetak*a9;
a20=thetak*a10;

a21=thetah*a11+deltah;
a22c=thetah*a12;         % note a22c is used to avoid confusion with a22
a23=thetah*a13;
a24=thetah*a14;
a25=thetah*a15;

a26=pic*a1;
a27=pic*a2;
a28=pic*a3;
a29=pic*a4 + thetah;
a30=pic*a5;



m211 = 0;  m212 = 0;  m213 =0;
m214 =1;  m215 = m212;  m216 = m213;
m221 = 0;  m222 = 0;  m223 =0;
m224 = m221;  m225 = 1;  m226 = m223;
m231 = 0;  m232 = 0;  m233 = 0;
m234 = m231;  m235 = m232;  m236 = 1;
m241 =a21;  m242 = a22c;  m243 = a23;
m244 = a24;  m245 = a25;  m246 = 0;
m251 = a16;  m252 = a17;  m253 = a18;
m254 = a19;  m255 = a20;  m256 = 0;
m261 = 0;  m262 = 0;  m263 = a22;
m264 = 0;  m265 = 0;  m266 = 0;
m2 = [m211 m212 m213 m214 m215 m216;  m221 m222 m223 m224 m225 m226;
      m231 m232 m233 m234 m235 m236;  m241 m242 m243 m244 m245 m246;
      m251 m252 m253 m254 m255 m256;  m261 m262 m263 m264 m265 m266];
clear m211 m212 m213 m214 m215 m216 m221 m222 m223 m224 m225 m226
clear m231 m232 m233 m234 m235 m236 m241 m242 m243 m244 m245 m246
clear m251 m252 m253 m254 m255 m256 m261 m262 m263 m264 m265 m266

m111 = beta*lambda*a1; m112 =beta*lambda'*a2; m113 =beta*lambda'*a3;
m114 = beta*(lambda'*a4 + deltah);  m115 = beta*lambda'*a5;  m116 =0;
m1row1 =[m111 m112 m113 m114 m115 m116];
m121 = gamma'*a26;  m122 =gamma*a27;
m123 = gamma*a28;  m124 = gamma*a29;
m125 = deltak' + gamma*a30;  m126 = 0;
m1row2 = beta*[m121 m122 m123 m124 m125 m126];
m131 = ub*lambda + ud'*a26 + ub*pic*a11;  m132 =ud*a27 + ub*pic*a12;
m133 = ud*a28 + ub*pic*a13 - ub*ub;  m134 = ud*a29 + ub*pic*a14;
m135 =ud*a30 + ub*pic*a15;  m136 = a22;
m1row3 = beta*[m131 m132 m133 m134 m135 m136];
m1 = [m1row1;  m1row2;  m1row3;
     eye(3) zeros(3)    ];
%clear m111 m112 m113 m114 m115 m116 m121 m122 m123 m124 m125 m126 m1row1
%clear m1row2 m131 m132 m133 m134 m135 m136 m1row3
%clear L1 L2 L3 L4 L5 L6 L7 L8 L9 L10 L11 L12 L13 L14 L15 L16 L17 L18 L19 L20
%clear L21 L22 L23 L24 L25 L26 L27 L28 L29 L30


%STEP 3
%Vaughan:  Choose costates so that the system lies in the stable subspace
%Note:     If the dimension of the stable subspace is LESS than the number
%          of state variables, then NO C.E. EXISTS (for generic x(0))
%          If the dimension of the stable subspace is GREATER than the number
%          of state variables, then there are many choices of the costates
%          that put the economy in the stable subspace, ie MULTIPLE C.E.



nx=3;


[W,D] = eig(sqrt(beta)*m2,m1);          % D = DELTA*sqrt(beta)  (see Ch.14)
diagD0 = diag(D);
dimsss = length(find(abs(diag(D))<1));  % dimension of the stable subspace
[diagD, index] = sort(abs(diag(D)));    % sort eigenvalues
W = W(:,index);  D=diag(diagD0(index)); % sort eigenvectors by eigenvalue

% Check eigenvalue split
if dimsss < nx
   error('No competitive equilibrium exists for generic initial conditions')
else
   W11 = W(1:nx,1:nx);  W12 = W(1:nx,(nx+1):2*nx);
   W21 = W((nx+1):2*nx,1:nx);  W22 = W((nx+1):2*nx,(nx+1):2*nx);
   D1 = D(1:nx,1:nx);  D2 = D((nx+1):2*nx,(nx+1):2*nx);
   M = W21/W11;               % costate "policy function"   mu(t) = M*x(t)
   if dimsss > nx;  disp('Warning!  Other competitive equilibria exist'); end
end
nk=1;nh=1;nz=1;nw=1;nd=1;nb=1;

%STEP 4:
%Compute the closed loop form of the equilbrium
%                   x(t+1) = a0*x(t) + c*w(t+1)

%V12 = inv(W21-(W22/W12)*W11);  V11 = -V12*W22/W12;
V = inv(W);  V11 = V(1:nx,1:nx);  V12 = V(1:nx,(nx+1):2*nx);
ao = W11*D1*(V11 + V12*M)/sqrt(beta);  a11o = ao(1:(nk+nh),1:(nk+nh));
c = [zeros(nk+nh,nw); c2];


%STEP 5:
%Display eigenvalue calculations for the user.  

%Endogenous eigenvalues.                                                       


   endo=eig(a11o);


%Exogenous eigenvalues.

   exo=eig(a22);

return
%STEP 6:
%Calculation of the solution matrices.                                      

    sh = ao(1:nh,1:nx);
    sk = ao(nh+1:nh+nk,1:nx);
    skl = [zeros(nk,nh) eye(nk,nk) zeros(nk,nz)];
    sd = [zeros(nd,nh+nk),ud];
    sb = [zeros(nb,nh+nk),ub];
    se = [zeros(nc,nh+nk),ue];
    si = [N1 N2 N3] + [N4 N5 zeros(ni,nz)]*M;
    sc = [N6 N7 N8] + [N9 N10 zeros(nc,nz)]*M;
    sg = [N11 N12 N13] + [N14 N15 zeros(ng,nz)]*M;
    ss = lambda*[eye(nh,nh) zeros(nh,nk+nz)] + pibar*sc;

disp('The matrix ao has been calculated for the law of ')
disp('motion of the entire state vector.  This matrix satisfies')
disp('        x(t+1) = ao*x(t) + c*w(t+1). ')
disp('The endogenous eigenvalues are in the vector endo,')
disp('and the exogenous eigenvalues are in the vector exog.')
disp('   ')
disp('   ')

disp('The solution to the model is given by c(t) = sc*x(t),')
disp(' g(t) = sg*x(t), h(t) = sh*x(t), i(t) = si*x(t)')
disp(' k(t) = sk*x(t), E(t) = se*x(t), and s(t) = ss*x(t).')
disp('The matrices sc, sg, sh, si, sk, se and ss have now been computed')
disp('and can be used in other matlab programs.')
disp('The matrices sb and sd are constructed so that b(t) = sb*x(t)')
disp('and d(t) = sd*x(t) and can be used in other matlab programs.')
%Calculating prices and Lagrange Multipliers%

    salpha = [N16 N17 N18] + [N19 N20 zeros(nd,nz)]*M;
    sp = phic'*salpha;
    sq = phii'*salpha;
    sw = -phig'*salpha;
    sr = gammak'*salpha;

    mh = [eye(nh) zeros(nh,nk+nz)]*M;
    mk = [zeros(nk,nh) eye(nk) zeros(nk,nz)]*M;
    mz = [zeros(nz,nh+nk) eye(nz)]*M;
    ms = sb - ss;

clear N1 N2 N3 N4 N5 N6 N7 N8 N9 N10 N11 N12 N13 N14 N15 N16 N17 N18 N19 N20










 
                                                                               






































 
disp('     ')
disp('     ')

disp('The price vectors satisfy alpha(t) = salpha*x(t), p(t) = sp*x(t),')
disp('q(t) = sq*x(t), w(t) = sw*x(t), r(t) = sr*x(t).')
disp('The shadow price vectors satisfy muh(t) = mh*x(t), muk(t) = mk*x(t),')
disp('muz(t) = mz*x(t), mus(t) = ms*x(t),')
disp('The matrices of these linear combinations can')
disp('be used in other matlab programs.')
disp('     ')
disp('     ')

disp('Your equilibrium has been calculated')
disp('     ')
