%DISTHET solves the competitive equilibrium of a distorted heterogeneous
% NOTE: EXPERIMENTALLY Changed, Jan13, 1993
%
%   economy. Two types of agents live in an economy with government and
%   externality.  Type i agent's problem is to maximize:
%
%     E{-0.5 SUM beta^t*[(si(t)-bi(t))*(si(t)-bi(t))+li(t)^2]}|I0  subject to:
%
%     si(t)=Lambdai1*hi(t-1)+Lambdai2*H1(t-1)+Lambdai3*H2(t-1)
%        +Pii1*ci(t)+Pii2*C1(t)+Pii3*C2(t)
%     hi(t)=Deltahi*hi(t-1)+DeltaHi1*H1(t-1)+DeltaHi2*H2(t-1)
%        +Thetahi*ci(t)+ThetaHi1*C1(t)+ThetahHi2*C2(t)
%     E{SUM beta^t*[(I+tauc)p(t)*ci(t)-(1-taul)w(t)*li(t)-alpha(t)*di(t)
%        -fi*(P1(t)+P2(t)-Ti(t)]}|I0-v0*k0i = 0
%
%   where si, hi, li, ci, Ti, Pi are consumption services, household capital
%   stock, labor, consumption, government transfer of type i agent and firms 
%   of type i's profit at t, i=1,2.  Capital letters denote aggregate 
%   variables.  tauj is tax on j, j=c,l,k,i.  Firms of type 1's problem is to
%   maximize expected profit:
% 
%E{SUM beta^t[p(t)[c(t)+E(t)]+q(t)*i(t)-r(t)*k(t-1)-alpha(t)*d(t)-w(t)*l(t)]}
%   
%   subject to:
%
%     Phic*(c(t)+E(t))+Phii*i(t)+Phig*g(t)=Gammak*k(t-1)+GammaK*K(t-1)+d(t)
%     g(t)*g(t)=l(t)^2
%
%   where c(t)=c1(t)+c2(t), similar for d(t) and l(t).  E(t) and g(t) are 
%   government spending and intermediate goods, respectively.  Firms of type 
%   2's problem is to maximize expected profit:
%
%     E{SUM beta^t[(I-tauk)r(t)*k(t-1)-(I+taui)q(t)*i(t)]}-v0*k0   subject to:
%
%     k(t)=Deltak*k(t-1)+DeltaK*K(t-1)+Thetahk*i(t)
%
%   where k0=k01+k02.  The state vector in this program is defined as
%   [z(t); z(t); h1(t-1); h2(t-1); k(t-1)].

% First Version, September 1, 1992.  Uses secant method to search
% for best mu02 and tauc, and doubling algorithm to compute the household's
% budget balances.  The equilibrium is computed by Vaughan's method.

%-------------- Part I -------------
% In this part, internal parameters are set and internal variables are
% defined.  

MAXIT=30;       % Maximum # of iterations
Tol=.00000001;       % Criterion for convergence
crit=1/sqrt(beta); % criterion for stable poles in L2 space
Success=1;
Notyet=0;
Status=Notyet;
vbeta=beta/(1-beta);

mu01=1; mu02=1; % mu02 and tt are the free parameter that we will search
tt=1;           % for optimal values using secant method below.  tt is
                % the proportional factor for tauc.
[nh, nc]=size(Thetah1);                 %
[nk, ni]=size(Thetak);                  % nh, nc, nk, ni, ns, nd, ng, nz
[ns,nh]=size(Lambda11);                 % nw, and nx are dimension of hi(t),
[nd, ng]=size(Phig);                    % ci(t), k(t), i(t), si(t), di(t),
[nz,nw]=size(C2);                       % g(t), z(t), w(t) and the state vector
nx=2*(nz+nh)+nk;                        % x(t), respectively.

sd1=[Ud1 zeros(nd,nx-nz)]; sd2=[zeros(nd,nz) Ud2 zeros(nd,nk+2*nh)];  

  % di(t)=sdi*x(t),where x(t) is the state vector

Lambda1=Lambda11+Lambda12; 
Lambda2=Lambda21+Lambda23; 
Pi1=Pi11+Pi12; Pi2=Pi21+Pi23;
Deltakb=Deltak+DeltaK;
Thetah1b=Thetah1+ThetaH11; Thetah2b=Thetah2+ThetaH22;
Gamma=Gammak+GammaK;
Uf=Ud1+Ud2-Phic*UE;   
nettauk=eye(nk)-tauk; nettaul=1-taul;

Phiin=inv([Phic'; Phig']);
Phi1=Phiin(:,1:nc); Phi2=Phiin(:,nc+1:nc+ng);
      
Tl=Phi2/nettaul;                  % Variables Tl, Pc, Pg and Tc, Tcc (below)
Pc=[eye(nc) zeros(nc,ng)]*Phiin'; %  are defined only for computational 
Pg=[zeros(ng,nc) eye(ng)]*Phiin'; %  conveniences.

z0=[mu02;tt];           % initializing the parameters to be adjusted.
                        % tt is the proportional factor of tauc
M11=[eye(nz) zeros(nz,2*nx-nz)]; M21=A22*M11;            % M11, through
M12=[zeros(nz) eye(nz) zeros(nz,2*(nx-nz))]; M22=A22*M12;% M29 are the "row"
M13=[zeros(nh,2*nz) eye(nh) zeros(nh, nx+nh+nk)];        % -like components of 
M14=[zeros(nh,2*nz+nh) eye(nh) zeros(nh, nx+nk)];        % the Vaughan's
M15=[zeros(nk,nx-nk) eye(nk) zeros(nk,nx)];              % equation system.
M26=[zeros(nz,nx) eye(nz) zeros(nz,nx-nz)];              % each M1i, M2i pair
M27=[zeros(nz,nz+nx) eye(nz) zeros(nz,2*nh+nk)];         % corresponds to
M20=[zeros(nk,2*nx-nk) eye(nk)];                         % an equation that's
M28=[zeros(nh,2*nz+nx) eye(nh) zeros(nh,nh+nk)];         % used to derive the
M29=[zeros(nh,2*nz+nh+nx) eye(nh) zeros(nh,nk)];         % Vaughan's system.
                                                   % More comments in Part III.
La=[eye(nc) eye(nc) zeros(nc, ng) Pc*Phii; zeros(ng,2*nc) eye(ng) Pg*Phii;];
NN1=[Pc*Uf zeros(nc,nz+2*nh) Pc*Gamma zeros(nc,nx);
     Pg*Uf zeros(ng,2*nh+nz) Pg*Gamma zeros(ng,nx);];
NN2=[Ub1 zeros(ns,nz) -Lambda1 -Lambda13 zeros(ns,nk+2*nz)];   %JAN13 !!!
NN3=-(eye(ni)+taui)\Thetak';
Ns1a=[zeros(ns,2*nz) Lambda1 Lambda13 zeros(ns,nx+nk)];
Ns2a=[zeros(ns,2*nz) Lambda22 Lambda2 zeros(ns,nx+nk)];
Nalphaa=[Pi11'*Ub1 zeros(nc,nz+nx) Thetah1' zeros(nc,nh+nk)];
M23a=[zeros(nh,2*nz) Deltah1+DeltaH11 DeltaH12 zeros(nh,nk+nx)];
M24a=[zeros(nh,2*nz) DeltaH21 Deltah2+DeltaH22 zeros(nh,nk+nx)];
M25a=[zeros(nk,nx-nk) Deltakb zeros(nk,nx)];
M16a=[-Ub1'*Ub1 zeros(nz,nx-nz) A22' zeros(nz,nx-nz)];
M17a=[zeros(nz) -Ub2'*Ub2 zeros(nz,nx-nz) A22' zeros(nz,nk+2*nh)];
M18a=[Lambda11'*Ub1 zeros(nh, nz+nx) Deltah1' zeros(nh,nk+nh)];
M19a=[zeros(nh,nz) Lambda21'*Ub2 zeros(nh,nh+nx) Deltah2' zeros(nh,nk)];
M10a=[zeros(nk,2*nx-nk) Deltak'];
%-------------------------The main loop -----------------------------------

for j=1:MAXIT
  del=diag(max(abs(z0)*1e-9,1e-9));     % del is perturbation on z0 for the
  del=[zeros(2,1) del];                 % purpose of secant searching.
  for jj=1:3            % jj=1 will calculate budget balances with z0, jj=2,3
    zz=z0-del(:,jj);    % will do the same calculations with perturbating
    mu02=zz(1);         % each of z0's components. -- used for secant's method
    mu=mu01+mu02;
    nettauc=eye(nc)+tauc*zz(2);
    Tc=Phi1/nettauc;  Tcc=Phii'*Tc*Pi11';

%-------- Part II:  Computing the expressions of ci(t), g(t) and i(t)------
% In this part, we use first order conditions of the economy to construct 
% equation L*[c1(t);c2(t);g(t);i(t)] = N*[x(t); mux(t)], that is, to express
% ci(t), g(t) and i(t) in terms of the state and costate variables.  Then
% [c1(t);c2(t);g(t);i(t)]=(L\N)*[x(t);mux(t)].

    L11=mu01*Pi21'*Pi22-mu02*Pi11'*Pi1;
    L12=mu01*Pi21'*Pi2-mu02*Pi11'*Pi13;
    L=[L11 L12 zeros(nc,ng+ni);                     
        La;
       Tcc*[Pi1 Pi13]/mu01 Phii'*Tl/mu zeros(ni)];     

    N13=mu02*Pi11'*Lambda1-mu01*Pi21'*Lambda22;
    N14=mu02*Pi11'*Lambda13-mu01*Pi21'*Lambda2;
    N=[-mu02*Pi11'*Ub1 mu01*Pi21'*Ub2 N13 N14 zeros(nc,nk+2*nz) ...
        -mu02*Thetah1' mu01*Thetah2' zeros(nc,nk);
        NN1;
        Tcc*NN2/mu01 Phii'*Tc*Thetah1'/mu01 zeros(ni,nh) NN3];
    N=L\N;    % [c1(t);c2(t);g(t);i(t)] = N*[x(t);mux(t)]. Thus 
              % c1(t) = Nc1*[x(t);mux(t)] with Nc1=N(1:nc,:) and so on.

%----Part III: Construction of the Vaughan-like linear equation system----
% From results in Part II, we go on to express all non-state variables in
% terms of state variables, and degenerate first order conditions into
% a set of equations that only state and costate variables are involved.
% Such a set of equations relates [x(t); mux(t)] with [x(t+1); mux(t+1)].
% and Vaughan's method can be applied.

 %Step I: Express si(t),alpha(t) in [x(t);mux(t)],as:s1(t)=Ns1*[x(t);mux(t)].

    Ns1=Pi1*N(1:nc,:)+Pi13*N(nc+(1:nc),:)+Ns1a;
    Ns2=Pi22*N(1:nc,:)+Pi2*N(nc+(1:nc),:)+Ns2a;
    Nalpha=Tc*(-Pi11'*Ns1+Nalphaa)/mu01-Tl*N(2*nc+(1:ng),:)/mu;

 % Step II: Now each M1i, M2i pair represents an equation of the form:
 % M1i*[x(t+1);mux(t+1)] = M2i*[x(t);mux(t)]

    % two pairs Mi1, Mi2, i=1,2 (defined above) represent z(t+1)=A22*z(t)

    M23=Thetah1b*N(1:nc,:)+ThetaH12*N(nc+(1:nc),:)+M23a;
    M24=Thetah2b*N(nc+1:2*nc,:)+ThetaH21*N(1:nc,:)+M24a;

  % two pairs Mi3, Mi4, i=1,2 represent hi(t)=Deltahibar*hi(t-1)+
  %     DeltaHij*hj(t-1)+Thetahibar*ci(t)+ThetaHji*cj(t)

    M25=Thetak*N(2*nc+ng+(1:ni),:)+M25a;

  % M15, M25 represent k(t)=Deltakb*k(t-1)+Thetak*i(t)

    M16=beta*(M16a+Ub1'*Ns1+Ud1'*mu01*Nalpha);
    M17=beta*(M17a+Ub2'*Ns2+Ud2'*mu02*Nalpha);

  % Muzi(t)=beta*{A22'Muzi(t+1)+Ubi'[si(t+1)-Ubi*z(t+1)]+mu0i*Udi'alpha(t+1)}

    M18=beta*(M18a-Lambda11'*Ns1);
    M19=beta*(M19a-Lambda21'*Ns2); 

  % Muhi(t)=beta*[Deltahi'*Muhi(t+1) + Lambdai1'*(bi(t+1)-si(t+1))]

    M10=beta*(nettauk*Gammak'*Nalpha+M10a);

  % Muk(t)=beta*[Deltak'Muk(t+1)+nettauk*Gammak'alpha(t+1)

    M1=[M11;M12;M13;M14;M15;M16;M17;M18;M19;M10]; 
    M2=[M21;M22;M23;M24;M25;M26;M27;M28;M29;M20]; 

% M1 and M2 are the Vaughan's matrices that satisfy: 
%        M1*[x(t+1);mux(t+1)]=M2*[x(t);mux(t)]

%--------part IV: Vaughan's Method to find stable solution----------------
% for more about Vaughan's method, please refer to Chapter XX of Hansen and
% Sargent: XXXXXXXXXXX

    [W,D]=eig(M2,M1);     % W: matirx of generalized eigenvectors
    diagV=diag(D);        % D: diagonal matrix of generalized eigenvalues
    if jj==1
      ds=sum(diagV<crit);
      if ds<nx            % stable subspace too small
        disp('WARNING: Competitive equilibrium may not exist')
      elseif ds>nx        % stable subspace too large
        disp('Warning: May exist other competitive equilibria')
      end
    end
    [absd,order]=sort(abs(diagV));     % arrange eigenvectors, eigenvalues in 
    W=W(:,order);                      % order of absolute values of 
    D=diag(diagV(order));              % eigenvalues.		
    V=inv(W);
    M=W(nx+1:2*nx,1:nx)/W(1:nx,1:nx); M=real(M);

	% M is the costate policy function: mux(t)=M*x(t)

    VV=V(1:nx,1:nx)+V(1:nx,nx+1:2*nx)*M;
    Ao=W(1:nx,1:nx)*D(1:nx,1:nx)*VV; Ao=real(Ao);
    C=[C2; C2; zeros(2*nh+nk,nw)]; C=real(C);

	% Ao and C are the closed loop solution of the states of the economy:
        % 	 x(t+1)=Ao*x(t)+C*w(t+1)

    sk=Ao(nx-nk+1:nx,:);          
    sc1=N(1:nc,1:nx)+N(1:nc,nx+(1:nx))*M;             % Among the solution
    sc2=N(nc+1:2*nc,1:nx)+N(nc+1:2*nc,nx+1:2*nx)*M;   % matrices, these are
    sg=N(2*nc+(1:ng),1:nx)+N(2*nc+(1:ng),nx+1:2*nx)*M;% pertinent for the
    md=Nalpha(:,1:nx)+Nalpha(:,nx+1:2*nx)*M;      % calculation of the
    mc=Phic'*md;                                  % households' budget
    mk=Gammak'*md;                                % constraints below.
    Muk=M(nx-nk+1:nx,:);            % Muk is the Lagrange multiplier of the 
                                    % constraint in firm 2's problem

%---------Part V: Calculating Household's Budget Constraints--------
    % If budget constraints not hold, change values of free parameters mu02 
    % and tt through secant method below and repeat the entire procedure.

        % computing the risk premium term in the budget balance

    v=vbeta*doublej2(beta*Ao,C,Ao,C);

        % computing the balance kernel Zi.  Household i's budget balance
        % at date t is: x(t)'*Zi*x(t).  (Assume Ti(t) always equal to 0)
    
    Za=(md'*GammaK+Muk'*DeltaK)*sk;
    Z1=mc'*nettauc*sc1+nettaul*mu01/mu*md'*Phig*sg-md'*sd1-f1*Za;
    Z2=mc'*nettauc*sc2+nettaul*mu02/mu*md'*Phig*sg-md'*sd2-(1-f1)*Za;

      % computing the budget balances of households of type 1 and 2

    balancea=x0'*(mk'*nettauk+Muk'*Deltak);
    balance1=x0'*doublej2(beta*Ao',Z1,Ao',1)*x0+sum(diag(Z1*v))-balancea*k01;
    balance2=x0'*doublej2(beta*Ao',Z2,Ao',1)*x0+sum(diag(Z2*v))-balancea*k02;
    balance(:,jj)=[balance1;balance2];

%--------- Part VI: secant method to find better values of mu02,tt-------
% Principly we search for new mu02 and new tauc.  But in general this leaves
% too many degrees of freedom since tauc may not be a scalar.  We thus 
% restrict our searching object to mu02 and tt, the proportional factor of
% tauc, although it's not neccesarily so.

    if jj==3
      for ii=1:2
        balancej(:, ii)=(balance(:,1)-balance(:,ii+1))/del(ii,ii+1);

          % approximate Jacobian of vector "balance(:,1)" with respect to z0

      end;
      z=z0-balancej\balance(:,1);     % Apply Newton's Method using the 
				      % approximate Jacobian.
      if max(abs(z-z0))<Tol
        Status=Success;		    % can successfully leave the loop now.
      end;
      z0=z;
    end;

  end;
  if Status==Success
    break;
  end    	
end;      %-----------end of the main loop-------------------

%-----------closing of the program ---------------

endo=eig(Ao(2*nz+1:2*(nz+nh)+nk,2*nz+1:2*(nz+nh)+nk));
exo=eig(A22);      % endogenous and exogenous eigenvalues of the system

sh1=Ao(2*nz+1:2*nz+nh,:);
sh2=Ao(2*nz+nh+(1:nh),:);
sb1=[Ub1 zeros(ns,nx-nz)];
sb2=[zeros(ns,nz) Ub2 zeros(ns,nk+2*nh)];
se=[UE zeros(nc,nx-nz)];
si=N(2*nc+ng+(1:ni),1:nx)+N(2*nc+ng+(1:ni),nx+1:2*nx)*M;
ss1=Ns1(:,1:nx)+Ns1(:,nx+1:2*nx)*M;
ss2=Ns2(:,1:nx)+Ns2(:,nx+1:2*nx)*M;
mi=Phii'*md;

mh1=M(2*nz+1:2*nz+nh,:);
mh2=M(2*nz+nh+1:2*(nz+nh),:);
ms1=sb1-ss1;
ms2=sb2-ss2;
mz1=M(1:nz,:);
mz2=M(nz+1:2*nz,:);
tauc=tauc*z0(2);
mc1=mc; mc2=mc; md1=md; md2=md;

clc
if Status==Success
  disp('The equilibrium has been successfully computed.')
elseif j>=MAXIT
  disp('WARNING: iteration limit is exceeded.')
end
disp(' ')
disp(' ')
disp('The following solution matrices have been computed:')
disp(' ')
disp('  - Ao and C for the law of motion for the state vector. They satisfy:')
disp(' ')
disp('          x(t+1) = Ao*x(t) + C*w(t+1).')
disp(' ')
disp('    where the state vector x(t) = [z(t); z(t); h1(t-1); h2(t-1); k(t-1)];')
disp('  - endo and exo which contain the endogenous and exogenous eigenvalues;')
disp('  - sy which express quantity y(t) as sy*x(t), for y = k, g, i, e ')
disp('    and ci, bi, si, hi, di, i=1,2;')
disp('  - my which express shadow prices My(t) as my*x(t) for y = ci, si, hi,')
disp('    di, i=1,2 and k, i;')
disp('  - Number of secant steps j;')
disp('  - Equilbrium shadow prices of wealth mu01 and mu02;')
disp('  - The final two households budget balances balance1 and balance2')
disp('  - taul, taui, tauk are not changed, but tax on consumption tauc')
disp('    that satisfies the government''s budget constraint are also computed.')

clear M11 M12 M13 M14 M15 M16 M17 M18 M19 M10 M21 M22 M23 M24 M25 M26
clear M27 M28 M29 M20 M10a M16a M17a M18a M19a M23a M24a M25a L11 L12
clear L N M1 M2 Tol MAXIT Success Notyet Status D La Lambda1 Lambda2
clear Thetah1b Thetah2b Deltakb nettaul nettauc Gamma  N13 N14 NN1 NN2
clear nd nc ng nk ni nh nz nw nx ns NN3 Nalpha Nalphaa Ns1 Ns1a Ns2 Ns2a
clear Uc Ug Phi1 Phi2 Phiin Tl Pc Pg Tc Tcc Ns1 Nalpha Ns2 absd order diagV
clear  balance Pi1 Pi2 Uf V VV W ii v   balancea balancej
clear jj crit del diagV ds mu nettauk tt v vbeta z zz z0
 

