										
function[K,S,ST]=doublex4(A,C,Q,R,c,sig)
% Initializes with po=I.  The version is designed to find the optimal
% stable work solution in, for example,  
% `permanent income' models that don't satisfy the detectability condition.
%  See AHMS (Computational economics) for details.  The program
%  adapts the algorithm to the risk-sensitive specicfication of
%  Hansen-Sargent (IEEE, May, 1995)  
%DOUBLEX4.M
%function[K,S,ST]=doublex4(A,C,Q,R,c,sig)
%  This program uses the "doubling algorithm" to solve the
%  Riccati matrix difference equations associated with the
%  undiscounted quadratic-Gauusian linear optimal control problems.
%  The control problem has the form
%
%    S(t) = min over u(t) of
%    x(t)' Q x(t) + u(t)' R u(t) + (2/sig)log E(t) exp (sig/2) S(x(t+1)),
%
%  subject to 
%
%     x(t+1) = A' x(t) + C' u(t) + c w(t+1),
% 
%  where w(t+1) is a Gaussian martingale difference sequence 
%  with unit covariance matrix. The program returns the steady
%  state  value function in S.  The optimal control law is
%  u(t) = -K' * x(t).  The program also returns ST, which
%  is the quadratic form in E(t) exp S(x(t+1)).
%  The matrices S and ST should be positive semi-definite


ss=max(size(A));
v=eye(ss);
po=v;

J=C'*(R\C)-sig*c*c';

%Ji=inv(v+J*po);
%a0=A'*Ji';
%b0=J'*Ji';          %  This is the J of Whittle.
%g0=Q-po+A'*Ji'*po'*A;


t=v+J*po;
a0=t\(A');
b0=t\J;
g0=Q-po+A*po*a0;

tol=1e-15;
dd=1;

while dd>tol

a1=a0*((v+b0*g0)\a0);
b1=b0+a0*((v+b0*g0)\(b0*a0'));
g1=g0+a0'*g0*((v+b0*g0)\a0);


%dd=max(max(abs(g1-g0)));

dd=norm(g1-g0)/norm(g0);

a0=a1;
b0=b1;
g0=g1;
end

gg1=g1+po;   %  gg1=g1+po % (changed from  paper)
g1t=(v-sig*gg1*c*c')\gg1;

k1=A*g1t*C'/(C*g1t*C'+R);

K=k1;S=gg1;ST=g1t;
