% erric2.m   version of ericc.m to accept time-varying A,B,Q,R from socsec2.m
% Outputs include F,P,Ao,phi
% Inputs must include beta,A,B,Q,R,C,sig,PT1,T,T1 (see socsec2.m and
% socall2.m which specify the various inputs.
%
%  Computes finite horizon linear regulator problem, for
%  matrices that change at retirment date T1.
%   Set beta=1, if there is no discounting.
%
%     ERICC2 calculates f_t of the feedback laws:
%		u_t = -f_t x_t
%  
%  that maximizes the function:
%  J(t) =
%  [x(t)'Rx(t) + u(t)'Qu(t)]+beta*(2/sig)log E(t)exp((sig/2) J(t+1)]
%  
%  subject to 
%               x[t+1] = Ax[t] + Bu[t] + C w[t+1]
%  J(T+1) =x(T+1)' PT1 x(T+1),
%  where x is the nx1 vector of states, u is the kx1 vector of controls,
%  A is nxn, B is nxk, Q is nxn, R is kxk, W is nxk. 
%  For t=1, ... , T, successive rows of P, Ao, and F have the
%  corresponding time t value functions, closed loops (A-BF)
%  and policy functions.
%  phi holds the `transition' functions
%     phi=[I a1 a2*a1  a3*a2*a1 ...  aT*a(T-1)* ... *a1]
%  where ai is the closed loop system at time i.  Successive
%  blocks of phi give the transition law from t-j to t.
%  Also computed are CC, II, KK, which contain the sequence
%  of decisions rules for c, i, and k; and meani, meanc, meank
%  which are the time series of means of i, c, and k.
%  The standard deviations of the distributions of c, i, and k
%  are stored in vc, vi, and vk.

m=max(size(A1));
[rb,cb]=size(B1);
% if nargin==7; W=zeros(m,cb); end;

AA=zeros(T*m,m);
BB=zeros(T*rb,cb);
II=zeros(T*ni,m);
CC=zeros(T*nc,m);
KK=zeros(T*nk,m);

%  edit the following lines to put in time-varying A's.

%for j=1:T

%  AA(m*(j-1)+1:j*m,:)=A;
%  BB(rb*(j-1)+1:j*rb,:)=B;

%end



  [mc,nC]=size(C1);

P=zeros(T*m,m);
F=zeros(cb*T,m);
Ao=zeros(T*m,m);
V=zeros(T*m,m);
phi=zeros(T*m,m);


% p0=-.01*eye(m);

  p0=PT1;

  phi=eye(m); 



for j=1:T


%  A= AA(m*(j-1)+1:j*m,:);
%  B= BB(rb*(j-1)+1:j*rb,:);

jj=T+1-j;
if jj > T1 
   R=R2;
   Q=Q2;
   A=A2;
   B=B2;
   C=C2;
   qi=Q2i; 
   s=S2;
   
   ud=ud2; 
 else
   R=R1;
   Q=Q1;
   A=A1;
   B=B1;
   C=C1;
   qi=Q1i;
   s=S1;
    
   ud=ud1;
end
 
p1=p0+sig*p0*C*((eye(nC)-sig*C'*p0*C)\C'*p0);  

%  The right hand side of the above is T(p0) of Hansen-Sargent

f0=(Q+beta*B'*p1*B)\(beta*B'*p1*A);

%  See Hansen-Sargent for this formula

p2=R+f0'*Q*f0+beta*(A-B*f0)'*p1*(A-B*f0);


%  p2=R + beta*A'*(p1 -beta*p1*B*((Q+beta*B'*p1*B)\B'*p1))*A;

%   alternative form for p2 (identical in output)

%  The right side of the above equation is D(p1) of Hansen and Sargent

p0=p2;

%  The following equations stack the outputs into grand matrices

   F(cb*(T-j)+1:cb*(T-j+1),:)=f0;

   P(m*(T-j)+1:m*(T-j+1),:)=p1;
                          ao=A-B*f0;
   Ao(m*(T-j)+1:m*(T-j+1),:)=A-B*f0;

  
%Calculation of the solution matrices.

    sh = ao(1:nh,1:nx);
    sk = ao(nh+1:nh+nk,1:nx);
    si = -f0 - qi*s;
    skl = [zeros(nk,nh) eye(nk,nk) zeros(nk,nz)];
    sd= [zeros(nd,nh+nk),ud];
    sb = [zeros(nb,nh+nk),ub];
    sc = phiinc*(-phii*si + gamma*skl + sd);
    sg = phiing*(-phii*si + gamma*skl + sd);
    ss = lambda*[eye(nh,nh) zeros(nh,nk+nz)] + pih*sc;

 
   II(ni*(T-j)+1:ni*(T-j+1),:) = si;
   CC(nc*(T-j)+1:nc*(T-j+1),:) = sc;  
   KK(nk*(T-j)+1:nk*(T-j+1),:) = sk;
    

end;
 
%  The following code creates a row matrix of transition functions

for j=1:T

    phi=[phi Ao(m*(j-1)+1:j*m,:)*phi(:,m*(j-1)+1:j*m)];

end 

meanc=zeros(T,1);
meani=zeros(T,1);
meank=zeros(T,1);
for j=1:T
    
    meanc(j)=(CC((j-1)*nc+1:j*nc,:)*phi(:,(j-1)*m+1:j*m))*x0';
    meani(j)=(II((j-1)*ni+1:j*ni,:)*phi(:,(j-1)*m+1:j*m))*x0';
    meank(j)=(KK((j-1)*nk+1:j*nk,:)*phi(:,(j-1)*m+1:j*m))*x0';


end

%  Calculate the unconditional state covariance matrices
%  The variance at time 2 (second period) is v0, which we put
%  in the first rows of V; the variance at time 3 is in the second set
%  of rows of V, and so on.
%  The variance at time 1 is taken to be zero; modify this if needed.
%
%  Note the formula that we use is
%
%       Var(1) =0  
%       Var(2) = C C'
%       Var(3) = C C' + Ao(2) Var(2) Ao(2)'
%       Var(j) = C C' + Ao(j-1) Var(j-1) Ao(j-1)' 
%
%  The variance of consumption at time j is
%
%       sc(j)* Var(j) sc(j)'
%
%  where sc(j) is the decision rule for consumption applying
%  at age j.
 
v0=C*C';
V(1:m,:)=v0;

for k=2:T
  j=k-1;
 
  at=Ao(m*(j-1)+1:j*m,:);

  V(m*(k-1)+1:k*m,:)=C*C' +at*v0*at';
  v0=C*C' +at*v0*at';

 end



%  Compute variances of consumption, investment, capital 

vc=zeros(T,1);
vk=zeros(T,1);
vi=zeros(T,1);
vc(1)=0;
 
  for j=2:T
   
   
   si =II(ni*(j-1)+1:ni*j,:);
   sc = CC(nc*(j-1)+1:nc*j,:); 
   sk = KK(nk*(j-1)+1:nk*j,:);

   vt=V(m*(j-1)+1:m*j,:); 

   vc(j)= sc*vt*sc';
   vi(j)= si*vt*si';

   vk(j)=sk*vt*sk';
   end

vc=sqrt(vc);   %  Compute standard deviations 
vi=sqrt(vi);
vk=sqrt(vk);  