% school2.m, revised from school 1 on Februrary 24, 2003
%  corrected a mistake in the "technology" description
%  
%  Version of Rosen's education model, with only one sector,
%  and upward supply curve of educated workers
% 
%  The model is
%
%  w_t = etad + alphad*N_t + cd*epsilon_dt
%  N_{t+4} = deltaN* N_{t+3} + n_t
%  v_t = beta^4 E_t \sum_{j=0}^\infty ((beta*deltaN)^j w_{t+4+j}
%  n_t = etas + alphas*v_t + cs*epsilon_st
%
%  where w_t is wage, N_t is stock of workers, v_t is the expected present
%  value of wages of a `freshman', n_t is the rate of new entrants,
%  epsilon_s and epsilon_d are mutually independent i.i.d. noises with
%  means of zero and unit variances.  Note that -etas/alphas is the intercept
%  of supply curve with v_t expressed as a function of n_t.  
%  
%  The model maps into the Hansen-Sargent framework with
%  Lambda =[alphad 0 0 0], h_{t-1}=[N_t n_{t-1} n_{t-2} n_{t-3}],
%  c_t = n_t, phic=[1 0]', phii=[1 alphas^{-1}]', phig=[1 -1]',
%  gamma=[0 0]', d_{1t} = etas+cs*epsilons, b_t = etad+cd*epsilond,
%  v_t = (alphad)^(-1)*mc*x_t, w_t = ms*x_t.  See Hansen-Sargent robustness
%  monograph, asset pricing chapter, for details.
%
%  Try an example with cs=0.  Because demand shock is a white noise,
%  n_t and v_t don't respond.  Current wages  absorb the entire shock.  See
%  the supply equation.  

%  Set k = 4.

% set parameters in Rosen's model
beta=1/1.05;
alphad=.1;  %slope of demand
etad=30;    %constant in demand curve for labor
rhod=0;     % a.r. coeffnt of demand shock
deltah1=.95

alphas=1; % slope of supply curve
etas=10;   % constant in supply curve of labor
rhos=0;    % a.r. coeffnt of supply shock  

cd=1   % std of demand shock
cs=1  % std of supply shock

sig1=0;    % use solvex both times
%sig2=-.5;  % this works with zero serial correlation

sig2=-.5;

eps1=.0000001;   % trick to get detectability
%eps1=0;  
%  because k=4, there are 3 lagged values of n

%  translate the parameters into hansen-sargent notation
lambda1=alphad;    % 
psi=1/alphas;

deltak=.90; thetak=0;

deltak=.90; thetak=0;
lambda=[lambda1 eps1 eps1 eps1];
%  pih2=2;    


pih=[0 ];


deltah=[deltah1 0 0  1; 0  0 0 0; 0 1  0 0; 0 0  1 0];
thetah=[0 ; 1  ; 0  ; 0 ];





phic=[1 ; 0];

phig=[ 0 -1]';



phii=[ -1 psi ]';

gamma=[ 0 0]';

%  ud=[5 1 0;  0 0];     %  first component of d is supply of new guys

ud=[etas 0 1;0 0 0];  %  set mean of etas  (supply side)


ub=[etad 1 0];

a22=[1 0 0; 0 rhos 0 ; 0 0 rhod];  % make the shocks iid
c2=[0 cd 0; 0 0 cs]';


deltak=.90; thetak=0;





%%%%%
  
%  Now compute the equilibrium

sig=0;
solvex

endo1=endo

nnc=6;   %  location of 1 in state vector

steadst
zs1=zs;

%  compute some means of prices

disp('mean wage w = '),ms*zs
disp('mean present value v = ' ), (lambda1)^(-1)*mc*zs
disp('mean of new entrants and stock of workers'),sc*zs, sh*zs


mv=(lambda1)^(-1)*mc  %  v_t = mv*x_t

sy=[ms;sc;mv;sh(1,:)];  %  wage,  new entries, v, N

x0=zs';
t1=80;

asimul;
figure(1),
subplot(2,2,1)
plot(y(:,1),'--b'),
hold on
plot(y(:,2),'-r'),title('wage(blue--) and new entries'),hold off
subplot(2,2,2),plot(y(:,3)),title('v')
subplot(2,2,3),plot(y(:,4)),title('N')

Ni=60;

figure(2)
z2=aimpulse(ao,c,sy,2,Ni); 
subplot(2,2,1),plot(z2(2:Ni,1)),title('wage to supply shock')
subplot(2,2,2),plot(z2(2:Ni,2)),title('new entries to sy shock')
subplot(2,2,3),plot(z2(2:Ni,3)),title('v to sy shock')
subplot(2,2,4),plot(z2(2:Ni,4)),title('N to sy shock')

figure(3)
z1=aimpulse(ao,c,sy,1,Ni);
subplot(2,2,1),plot(z1(2:Ni,1)),title('wage to dd shock')
subplot(2,2,2),plot(z1(2:Ni,2)),title('new entries to dd shock')
subplot(2,2,3),plot(z1(2:Ni,3)),title('v to dd shock')
subplot(2,2,4),plot(z1(2:Ni,4)),title('N to dd shock')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
%Now study the robust labor market

%return


sig=sig2;  

solvex


nnc=6;   %  location of 1 in state vector

steadst
zsr=zs;

%  compute some means of prices

disp('mean wage w = '),ms*zs
disp('mean present value v = ' ), (lambda1)^(-1)*mc*zs
disp('mean of new entrants and stock of workers'),sc*zs, sh*zs


mv=(lambda1)^(-1)*mc  %  v_t = mv*x_t

sy=[ms;sc;mv;sh(1,:)];  %  wage,  new entries, v, N

x0=zs';
t1=80;

asimul;
yr=y;
figure(4),
subplot(2,2,1)
plot(yr(:,1),'--b'),
hold on
plot(y(:,2),'-r'),title('wage(blue--) and new entries'),hold off
subplot(2,2,2),plot(y(:,3)),title('v')
subplot(2,2,3),plot(y(:,4)),title('N')

figure(5)
zr2=aimpulse(ao,c,sy,2,Ni); 
subplot(2,2,1),plot(zr2(2:Ni,1)),title('wage to supply shock')
subplot(2,2,2),plot(zr2(2:Ni,2)),title('new entries to sy shock')
subplot(2,2,3),plot(zr2(2:Ni,3)),title('v to sy shock')
subplot(2,2,4),plot(zr2(2:Ni,4)),title('N to sy shock')

figure(6)
zr1=aimpulse(ao,c,sy,1,Ni);
subplot(2,2,1),plot(zr1(2:Ni,1)),title('wage to dd shock')
subplot(2,2,2),plot(zr1(2:Ni,2)),title('new entries to dd shock')
subplot(2,2,3),plot(zr1(2:Ni,3)),title('v to dd shock')
subplot(2,2,4),plot(zr1(2:Ni,4)),title('N to dd shock')


endor=endo


[lambdah,pihh]=canonpr(beta,lambda,pih,deltah,thetah)

zr2(1:10,:)-z2(1:10,:)

figure(7)
subplot(2,2,1),plot(z2(2:Ni,1),'-b'),title('W to u_s','Fontsize',14)
hold on
    
subplot(2,2,1),plot(zr2(2:Ni,1),'--r')
hold off 
subplot(2,2,2),plot(z2(2:Ni,2),'-b'),title('n to u_s','Fontsize',14)
hold on
subplot(2,2,2),plot(zr2(2:Ni,2),'--r')
hold off
subplot(2,2,3),plot(z2(2:Ni,3),'-b'),title('v to u_s','Fontsize',14)
hold on
subplot(2,2,3),plot(zr2(2:Ni,3),'--r')
hold off
subplot(2,2,4),plot(z2(2:Ni,4),'-b'),title('N to u_s','Fontsize',14)
hold on
subplot(2,2,4),plot(zr2(2:Ni,4),'--r')
hold off

figure(7)
print -depsc rosenrob1
