
% seasapp1.m   Testing, nonfunction version of seasapp.m
load  hseas

%  SEASAPP1.M
%  Uses frequency domain method to implement a version of the
%  Hansen-Sargent formula for the linear least squares forecast
%  of a geometric distributed lead of a univariate arma process.
%
%  The program prompts the user for a and b, both of which must be
%  vectors of the same length.  a=[a0 ... am], b=[b0 ... bm],
%  where
%       Sum aj*x(t-j) = Sum bj*a(t-j)
%
%  where {a(t)} is a white noise process with unit variance.  The program
%  also prompts the user for lam, which is the decay factor in the
%  geometric distributed lead
%     E Sum (lam)^j * x(t+j) = Sum bb(j) * x(t-j)
%
%  The program returns the vector bb=[bb0 ... bbr].
%
%  We enter a(L) and b(L) as polynomials, pad them out with 100
%  zeros, then take their fft to get a(w) and b(w).  We also
%  form lam(w) using this method.  Then we form
%  b(w)/(a(w)*lam(-w)), and take its inverse fft.  Then we lop off
%  the coefficients (n+1:2n), which correspond to negative powers of L
%  and take the fft of the result.  We then multiply this by
%  a(w)/b(w) and inverse fft to obtain the final "Hansen-Sargent" coefficients.
%a=input('give a ')
%b=input('give b ')
%  lam=input('give geometric term lamda ')
%  the value of n must be in storage  %set this at the same value as n in seasadj

%a=[a';zeros(n,1)];
%b=[b';zeros(  n,1)];
%fa=fft(a);
%fb=fft(b);
%fr=fb./fa;  

a=[1 -.4 0 0 -.5 .3];
a =[1.0000   -0.6621    0.2654   -0.0000   -0.2654   -0.0000    0.2654 ...
  -0.0000   -0.2037];
b=[1 0];
fr=freq(b,a,n);  

s=fr.*conj(fr);
%semilogy(s)       %  spectral density of the raw series is in s

ns=length(a);
%h=zeros(1,1:ns)';
%h(1:ns/2+1)=ones(ns/2+1,1);
%h(ns/2+2:ns)=h(ns/2-1:-1:1);    % h is the `adjustment filter'
%h=h.';
                             % read in h from having run seasadj
xh=fft(h);

g=xh.*conj(xh).*s;     %  spectral density of adjusted series
%g=g.';
%[bw,se]=factor(g');      % get Wold representation for the s.a. process

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  include the guts of factor.m
%function[bw,se]=factor(s)
%  function[b,se]=factor(s)
%
%  This program accepts input in form of spectral density s
%  of a univariate process, and returns the kernel of the Wold representation
%  for the process in b.  The one step ahead prediction
%  error variance is in se.
%  The method is described in Whittle's Prediction and Regulation
%  (2nd ed, U of Minn Press, 1983) on page 26.
%
%  We take log(s), call it ls.  Then we take the inverse fft
%  of ls, call it cc.  The one-step ahead prediction error is given
%  by the Wiener-Kolmogorov formula se=exp(cc(1)).  We take the
%  strictly positive powers of cc, pad with zeros, take the fft of
%  the result, take its exponential, then finally inverse fft to get
%  the moving average kernel, which is stored in b.  The interesting
%  outputs are se, the one-step ahead prediction variance, and
%  b, the kernel in the moving average representation.
%
%  SEE ALSO: FACTORF.M, FORECASF.M
ss=g';                   %note that s is being reset
ls=log(ss);
cc=ifft(ls);
n0=max(size(cc));
n02=n0/2;
c1=cc(2:n0/2+1);
se=exp(cc(1));
p1=[0;c1;zeros(n02,1)];
pf=fft(p1);
bf=exp(pf);
bbw=ifft(bf);
bw=real(bbw);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   this is end of factor.m






nb=size(bw);              %  bw is the Wold m.a. kernel for the adjusted process
%return
%lam = .95       %  setting for lambda
ns=n;
llam=[1 -lam]';
llam=[llam;zeros(ns-2,1)];
flam=fft(llam);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Now we'll compute the transfer fun for the s.a. series

fbb=fft(bw(1:nb-1));
frat=fbb./conj(flam);
gh=ifft(frat);
hh=real(gh);
no=max(size(hh));
hhh=hh;hhh(no/2+2:no)=zeros(no/2-1,1);
%plot(hhh),title('coeffs on innovs to y for forming geom sum'),pause
fhhh=fft(hhh);
s=abs(fhhh);nn=max(size(s));f=2*pi*(0:nn-1)/nn;
%semilogy(f,s),title('freq response of geom sum'),pause
%fr=fr.';
nf=length(bf);   %  bf is the freq response for the m.a. kernel
k=ifft(fhhh./bf(1:nf-1));   %  the k's are the coefficients
                           % and fhhh./bf(1:nf-1) is the freq response
                           % for the adjusted data
ksa=real(k);				%  coefficients for adjusted data
kfrsa=fhhh./bf(1:nf-1);   	%  freq response for adjusted data

%kk=real(k);
%plot(kk),title('coefficients on y s for forming geom sum'),pause




fbbs=se*fbb.*conj(fbb);
app=(abs((kfrnsa-kfrsa)).^2).*fbbs;
appro=sum(app);
semilogy([abs(kfrnsa) abs(kfrsa)])
