% TEST2.M
%
%  This program prices two assets in a Lucas tree economy.
%  The two assets are the Lucas tree and another asset that has a return that
%  is perfectly negatively correlated with the Lucas tree.  The program
%  also prices a one-period "real bill".
%
%  Preferences are
%
%    -.5 E Sum beta^t [(c(t) - 30 )^2 ] | I(0)
%
%  The technology is c(t) = d(t).
%  The dividend process is
%
%      d(t) = 5 + d1(t) + d2(t)
%  where d1(t) and d2(t) are first order a.r. processes with
%  a.r. coefficients .8 and -.5, respectively, and with innovation
%  variances .25 and 1, respectively.
%
%  Simulations of the asset prices are stored in yy.  The returns on the
%  two assets and on the real bill  are stored in xx
%  The riskpremia associated with the two assets are stored in riskpr1 and
%  riskpr2.
%  The asset-pricing matrices (quadratic form terms) are stored in v21 and
%  v22 for assets 1 and 2.
gam=0;
gamma=[gam 0]'
phic=[1 0]'
phig=[0 1]'
phi1=.00001
phii=[0 -phi1]'
deltak=.95
thetak=1
beta=1/1.05
ud=[5 1 0; 0 0 0]
a22=[1 0 0; 0 .8 0; 0 0 -.5]
c2=[0 .5 0; 0 0 1]'
lambda=0;
pih=1
deltah=.9
thetah=(1-deltah)
ub=[30 0 0]
x0=[0 0 1 0 0]
% Now we'll do the programming required to compute an equilibrium
solvea
% Now we'll display aspects of the equilibrium that have been requested.
ao
c
sc
sh
sd
%  Now we prepare to do our simulations.  We set parameters to govern
%  the length of the simulations in asimul and asseta.
nt=100;
t1=100;
sy=[sc;sd]
% Now we'll simulate the economy and will store the data for consumption
% and the endowment shock in "data".  The first column is consumption.
rand('normal')
rand('seed',99)
clg
asimul
data=y;     % consumption and endowment and zero.
% Now we'll price the "Lucas tree".  We must set the payout matrix as required.
% Type "help asseta" to see the parameters that we must specify.
% We use asseta to compute the asset price.
pay=[0 0 5 1 .1]
asseta
v21=v2              % Stores the matrix in the quadratic form in asset pricing
		    % formula for the first asset
riskpr1=riskpr     % Stores the riskpremium parameter for this asset
z1=y;
%  The third column of z1 stores the asset price while the fourth stores the
%  gross return.
%  Now we set the vector "pay" to price the other asset asked for in the exam.
pay=[0 0 5 -1 -.1]
asseta
riskpr2=riskpr      % Stores the riskpremium parameter for this asset
v22=v2              % Stores the matrix in the quadratic form in asset pricing
		    % formula for the second asset
z2=y;
%  The third column of z2 stores the asset price while the fourth stores the
%  gross return.
r1=R1(1:99)';       % The gross one-period sure rate of return
										
xx=[z1(:,4),z2(:,4),r1.^(-1)]
yy=[z1(:,3),z2(:,3)]
riskpr1
riskpr2
v21
v22
mc
plot(data(:,1)),title('consumption'),pause
plot(yy),title('asset prices'),pause
plot(xx(:,1:2)),title('returns on two assets'),pause
mean(xx)             % mean returns on first asset, second asset, and one
		     % period sure loan ('real bill')
mean(yy)
